@@ -6,7 +6,7 @@
  * Provides the client half of the process.  Should be considered
  * untrusted by the privman server.
  *
- * $Id: priv_client.cc,v 1.38 2002/11/12 04:43:50 dougk Exp $
+ * $Id: priv_client.cc,v 1.39 2003/04/10 00:25:52 dougk Exp $
  */
 
 #include "../config.h"
@@ -43,8 +43,6 @@
 #define UNIX_PATH_MAX 108
 #endif
 
-extern char **environ;
-
 #ifdef HAVE_LIBPAM
 /* Used as a cache for get/set_item */
 static const void      *pam_types[20] = { 0 };
@@ -493,6 +491,7 @@ static int priv_pam_simple_func(pam_handle_t *pamh, int flags,
         rc = msg_getInt(msg);
         if (rc < 0) {
             errno = -rc;
+            msg_delete(msg);
             return PAM_PERM_DENIED;
         }
 
@@ -569,6 +568,7 @@ int priv_pam_set_item(pam_handle_t *pamh, int item_type, const void *item)
     rc = msg_getInt(msg); /* were we denied at the gate? */
     if (rc < 0) {
         errno = -rc;
+        msg_delete(msg);
         return PAM_PERM_DENIED;
     }
 
@@ -619,6 +619,7 @@ int priv_pam_get_item(pam_handle_t *pamh, int item_type, const void **item)
     rc = (enum privman_responces)msg_getInt(msg);
     if (rc < 0) {
         errno = -rc;
+        msg_delete(msg);
         return PAM_PERM_DENIED;
     }
     assert((enum privman_responces)rc == PRIV_PAM_RC);
@@ -964,151 +965,87 @@ pid_t	priv_wait4(pid_t pid, int *status, int options, struct rusage *rus)
 }
 
 
-/* Table needed for popen.  We have to be able to map a handle to 
- * a File Descriptor.. because CMD_PCLOSE requires the handle that
- * POPEN_AS returned so that it knows which process is expected to end.
- *
- * Maps FILE* to pid.
- */
-static std::map<FILE*, int> file_pid_map;
-/* 
- * popen, as the specified user in the specified chroot.
- * Er, as this is on top of rerun, the level of ganularity of
- * control may be inadaquate.
- */
-static void priv_popen_impl(char * const arg[])
-{
-    int                 i, fd;
-    struct sockaddr_un  saddr;
-    char               *argv[] = { "sh", "-c", arg[0], 0 };
-    char               *type;
-
-    /* 1. Close all fd's except for stdin,out,err.                      */
-    for (i = 3; i < 255; ++i)
-        close(i);
-
-    /* 2. Connect to the unix domain socket provided.                   */
-    memset(&saddr, 0, sizeof(saddr));
-    saddr.sun_family = AF_LOCAL;
-    memcpy(saddr.sun_path, arg[2], strlen(arg[2])+1);
-
-    if ((fd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
-        boom("priv_popen_as(socket)");
-    }
-    if (connect(fd, (struct sockaddr*)&saddr, sizeof(saddr)) < 0) {
-        boom("priv_popen_as(connect)");
-    }
-
-    /* 3. Dup the socket onto the fd requested.  Close the socket       */
-    type = arg[1];
-    if (type[0] == 'w') {
-        dup2(fd, STDIN_FILENO);
-    } else if (type[0] == 'r') {
-        dup2(fd, STDOUT_FILENO);
-    }
-    close(fd);
-
-    /* 4. execve.                                                       */
-    execve("/bin/sh", argv, environ);
-    _exit(-1);
-}
-
+static std::map<int, int> fd_handle_map; /* map fd to popen handle */
 
 FILE*   priv_popen_as(const char *command, const char *type, const char *user)
 {
-    char               *socketname;
-    int                 sock, i;
-    struct sockaddr_un  saddr;
-    pid_t               pid;
-    FILE               *f;
+    message_t  *msg = msg_new();
+    FILE       *retval;
+    int         fd;
+    int         rc;
 
-    /* 0.  Stupid checks.                                               */
-    if (type[1] != '\0' || (type[0] != 'r' && type[0] != 'w')) {
+    /* Stupid checks */
+    if (command == NULL || type == NULL || type[1] != '\0' ||
+            (type[0] != 'r' && type[0] != 'w')) {
         errno = EINVAL;
         return NULL;
     }
+    msg_init(msg, CMD_POPEN);
 
-    /* 1.  Create a socket name random-nuf.
-     * Bad.  RACEABLE.  I'd use anonymous, but they don't appear to be
-     * universally supported.  This is why I don't support chroot.
-     */
-    
-    do {
-        if ((sock = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0)
-            boom("priv_popen_as(socket)");
+    msg_addString(msg, command);
+    if (type[1] == 'r') {
+        msg_addInt(msg, 0);
+    } else {
+        msg_addInt(msg, 1);
+    }
+    msg_addString(msg, user);
+    msg_addString(msg, "/");
 
-        socketname = tempnam(0, "privmn");
+    msg_sendmsg(msg, privmand_fd, "priv_popen(sendmsg)");
+    msg_clear(msg);
 
-        memset(&saddr, 0, sizeof(saddr));
-        saddr.sun_family = AF_LOCAL;
-        memcpy(saddr.sun_path,socketname,strlen(socketname)+1);
+    msg_recvmsg(msg, privmand_fd, "priv_popen(recvmsg)");
 
-        i = bind(sock, (sockaddr*)&saddr, sizeof(saddr));
+    rc = msg_getInt(msg);
 
-        if (i < 0)
-            close(sock);
-
-    } while (i == -1 && errno == EINVAL);
-    if (i < 0)
-        boom("priv_popen_as(bind)");
-
-    /* And chmod it 666 */
-    chmod(saddr.sun_path, 0666);
-
-    if (listen(sock, 1) < 0)
-        boom("priv_popen_as(listen)");
-
-    /* 2.  Rerun_as.  Args are command, type, and filename.           */
-
-    char * const args[4] = { (char*)command, (char*)type, socketname, 0};
-
-    pid = priv_rerunas(priv_popen_impl, args, user, NULL,
-            PRIV_RR_OLD_SLAVE_MONITORED);
-    if (pid < 0) {
-        close(sock);
-        unlink(socketname);
-        return NULL;
+    if (rc < 0) {
+        errno = -rc;
+        retval = NULL;
+    } else {
+        fd = msg_getFd(msg);
+        retval = fdopen(fd, type);
+        fd_handle_map[fd] = rc;
     }
 
-    /* 3.  Get fd.  Close off socket since we are not listening anymore. */
-    int         fd;
-    if ((fd = accept(sock, 0, 0)) < 0) {
-        perror("accept failed");
-        boom("priv_popen_as(accept)");
-    }
-
-    close(sock);
-    unlink(socketname);
-
-    /* 4.  Turn it into a FILE* and add to the map.                     */
-    f = fdopen(fd, type);
-    file_pid_map[f] = pid;
-
-    /* 5.  Done.                                                        */
-    return f;
+    msg_delete(msg);
+    return retval;
 }
 
 int     priv_pclose(FILE *stream)
 {
-    int                 status, i;
-    /* 1. Get pid from map, and remove from map                         */
-    pid_t               pid = file_pid_map[stream];
-    file_pid_map.erase(stream);
+    message_t  *msg;
+    int         rc;
+    int         fd = fileno(stream);
+    int         handle;
 
-    /* 2. fclose() the stream.  This may cause the process to keel over
-     * but that's a good thing.                                         */
-    fclose(stream);
+    /* Stupid check */
+    if (fd_handle_map.count(fd) == 0)
+        return -1; /* EINVAL */
 
-    /* 3. Collect the status.  Its a grandchild of our parent so having
-     * the parent wait for it should work.                              */
-    do {
-        i = priv_wait4(pid, &status, 0, NULL);
-    } while (i == -1 && errno == EINTR);
+    handle = fd_handle_map[fd];
+    /* Close the stream, maybe causing the other side to keel over. */
+    fd_handle_map.erase(fd);
+    pclose(stream);
 
-    return status;
+    msg = msg_new();
+    msg_init(msg, CMD_PCLOSE);
+    msg_addInt(msg, handle);
+    msg_sendmsg(msg, privmand_fd, "priv_pclose(sendmsg)");
+
+    msg_clear(msg);
+    msg_recvmsg(msg, privmand_fd, "priv_pclose(recvmsg)");
+
+    rc = msg_getInt(msg);
+
+    if (rc < 0) {
+        errno = -rc;
+        rc = -1;
+    }
+
+    msg_delete(msg);
+    return rc;
 }
 
-
 /*
  * Extension framework. 
  */
