@@ -6,7 +6,7 @@
  * Header file for the implimentation bits of privman.  Things common
  * between the server and the client halves.
  *
- * $Id: priv_impl.h,v 1.21 2002/11/12 04:43:51 dougk Exp $
+ * $Id: priv_impl.h,v 1.22 2003/04/10 00:25:52 dougk Exp $
  */
 
 
@@ -43,6 +43,9 @@ enum commands {
     CMD_DAEMON                  = 'd',
     CMD_WAIT4                   = '4',
 
+    CMD_POPEN                   = 'k',
+    CMD_PCLOSE                  = 'K',
+
     CMD_RERUN_AS                = 'R',
     CMD_RESPAWN_AS              = 'r',
 
