@@ -6,7 +6,7 @@
  * Provides the server half of the process.  This half retains priviledge
  * and should be nei-invulnerable.
  *
- * $Id: privman.cc,v 1.56 2003/03/08 06:02:58 dougk Exp $
+ * $Id: privman.cc,v 1.57 2003/04/10 00:25:52 dougk Exp $
  */
 #include "../config.h"
 #include <stdio.h>
@@ -58,6 +58,8 @@
 #define UNUSED(p) p __attribute((unused))
 #endif
 
+extern char **environ;
+
 /* Globals */
 int             privmand_fd     = -1;    /* FD to talk to the other process */
 
@@ -677,7 +679,7 @@ static void forkProcess(message_t *msg) {
 
     n = fork();
     if (n < 0)
-        boom("forkProcess(fork))");
+        boom("forkProcess(fork)");
     else if (n > 0) {
         /* Parent.  Close the FD's */
         close(fds[1]);
@@ -689,6 +691,115 @@ static void forkProcess(message_t *msg) {
     }
 }
 
+/* Table needed for popen.  We have to be able to map a handle to 
+ * a File Descriptor.. because CMD_PCLOSE requires the handle that
+ * POPEN_AS returned so that it knows which process is expected to end.
+ *
+ * Maps fd to pid.
+ */
+static std::map<int, pid_t> file_pid_map;
+
+/* Static function used for rerunas pointer. */
+static void priv_popen_impl(char * const arg[])
+{
+    char       *argv[] = { "sh", "-c", arg[0], 0 };
+
+    /* fd already set up. */
+    execve("/bin/sh", argv, environ);
+    _exit(-1);
+}
+
+
+static void popenImpl(message_t *msg) {
+    char       *command;
+    int         type; /* 0 or 1, write, or read */
+    char       *user;
+    char       *root;
+    int         i;
+    int         fds[2];
+    int         pid;
+
+    /* 0: Get the args. */
+    command = msg_getAllocStr(msg, 4096, "popenImpl(bad command)");
+    type = msg_getInt(msg);
+    user = msg_getAllocStr(msg, 32, "popenImpl(bad user)");
+    root = msg_getAllocStr(msg, PATH_MAX+1, "popenImpl(bad chroot)");
+    if (type < 0 || type > 1)
+        boom("popenImpl(bad type)");
+
+    /* MAC check */
+    if (!runasPerm(user)) {
+        sendEPERM(msg, "Unauthorized rerunAs target");
+        goto cleanup;
+    }
+
+    /* 1: Create the shared fds. */
+    if (socketpair(AF_LOCAL, SOCK_STREAM, 0, fds) < 0)
+        boom("popenImpl(socketpair)");
+
+    /* 2: Fork. */
+    pid = fork();
+    if (pid < 0)
+        boom("popenImpl(fork)");
+    else if (pid == 0) { 
+        /* Just to make sure. */
+        close(privmand_fd);
+        privmand_fd = -1;
+        /* Close all non shared fds */
+        for (i=255 ; i > 3; --i) {
+            if (i != fds[0])
+                close(i);
+        }
+        if (type)
+            dup2(fds[0], STDOUT_FILENO);
+        else
+            dup2(fds[0], STDIN_FILENO);
+        /* execute shell .  This won't return since priv_popen_impl won't */
+        setup_child( priv_popen_impl, &command, user, root);
+    }
+    /* 4: Parent, save pid and fd in map for future pclose. */
+    close(fds[0]);
+    file_pid_map[fds[1]] = pid;
+
+    /* return fd */
+    msg_clear(msg);
+
+    msg_initResponce(msg, fds[1]); /* Pass as a handle. */
+    msg_setFd(msg, fds[1]);
+    msg_sendmsg(msg, privmand_fd, "popenImpl(sendmsg)");
+    close(fds[1]); /* We dont' need it anymore */
+cleanup:
+    free(command);
+    free(user);
+    free(root);
+    
+}
+
+static void pcloseImpl(message_t *msg) {
+    int         fd;
+    int         pid;
+    int         rc;
+
+    /* Get args. */
+    fd = msg_getInt(msg);
+    msg_clear(msg);
+    if (file_pid_map.count(fd) == 0) {
+        msg_initResponce(msg, -EPERM);
+        syslog(LOG_NOTICE, "%s", "pcloseImpl(bad handle)");
+    } else {
+        /* Get pid from map, remove from map. */
+        pid = file_pid_map[fd];
+        file_pid_map.erase(fd);
+        /* Collect the status.  ie, Blocking. */
+        if (wait4(pid, &rc, 0, NULL) < 0)
+            rc = -EINVAL;
+
+        msg_initResponce(msg, rc);
+    }
+    msg_sendmsg(msg, privmand_fd, "pcloseImpl(sendmsg)");
+}
+
+
 static void daemonProcess(message_t *msg) {
     /* Daemon.
      * 1) Fork, parent quits.  No mucking with global state
@@ -904,6 +1015,8 @@ static bool validRequest(enum commands c) {
         return true; 
     case CMD_RERUN_AS:
     case CMD_RESPAWN_AS:
+    case CMD_POPEN:
+    case CMD_PCLOSE:
         return config->rerunas;
     case CMD_BIND:
         return !config->bind_port.empty();
@@ -1014,6 +1127,9 @@ void privman_serv_init(void)
     function_map[ CMD_DAEMON            ] = daemonProcess;
     function_map[ CMD_WAIT4             ] = privWait4;
 
+    function_map[ CMD_POPEN             ] = popenImpl;
+    function_map[ CMD_PCLOSE            ] = pcloseImpl;
+
     function_map[ CMD_RERUN_AS          ] = rerunAsProcess;
     function_map[ CMD_RESPAWN_AS        ] = respawnAsProcess;
 
