@@ -7,7 +7,7 @@
  * out.  Its is far more complex than it needs to be (fork?  Please.) in
  * order to test additional Privman functionality.
  *
- * $Id: mycat2.c,v 1.4 2002/10/30 17:30:35 dougk Exp $
+ * $Id: mycat2.c,v 1.5 2002/11/01 16:22:27 dougk Exp $
  */
 
 #include "../config.h"
@@ -25,7 +25,8 @@ int main(int argc, char *argv[])
     char buf[4096];
     char *cwd;
 
-    cwd = getcwd(0,0); /* priv_init does a chdir("/") */
+    if ((cwd = getcwd(0,1024)) == NULL) /* priv_init does a chdir("/") */
+        perror("getcwd");
 
     /* Reuse the one config file */
     priv_init("mycat");
