@@ -8,7 +8,7 @@
  * connection, which ought to be "test passed."  It only listens
  * for one connection, and quits upon seeing ^D
  *
- * $Id: myecho.c,v 1.12 2002/10/30 21:34:52 dougk Exp $
+ * $Id: myecho.c,v 1.15 2002/11/04 15:55:58 dougk Exp $
  */
 
 #include "../config.h"
@@ -23,6 +23,8 @@
 
 #include <netdb.h>
 
+#include <errno.h>
+
 #include <netinet/in.h>
 
 /* ntohs */
@@ -60,11 +62,25 @@ int getPort(char *port) {
     return (int)retval;
 }
 
+int my_socket(char * const args[])
+{
+    static int handled = -1; /* Allow 2 calls. */
+    int sock;
+    if (handled < 1) {
+        sock = socket(PF_INET, SOCK_STREAM, 0); /* default protocol */
+        ++handled;
+    } else {
+        errno = EPERM;
+        sock = -1;
+    }
+    return sock;
+}
+
 int main(int argc, char *argv[])
 {
     int sock, port, connection;
     int sock2;
-    int n;
+    int n, my_socket_handle;
 
     struct sockaddr_in  addr;
     struct sockaddr_in  client;
@@ -72,7 +88,9 @@ int main(int argc, char *argv[])
     int                 clientlen = sizeof(client);
 
     char                buf[4096];
+    char * const        args[1] = { NULL }; 
 
+    my_socket_handle = priv_register_cap_fn(my_socket);
     priv_init("myecho");
 
     if (argc < 2)
@@ -86,7 +104,7 @@ int main(int argc, char *argv[])
         exit(EXIT_FAILURE);
     }
 
-    sock = socket(PF_INET, SOCK_STREAM, 0); /* default protocol */
+    sock = priv_invoke_cap_fn(my_socket_handle, args);
     if (sock < 0) {
         perror("myecho(socket)");
         exit(EXIT_FAILURE);
@@ -112,7 +130,7 @@ int main(int argc, char *argv[])
     }
 
     /* server half.  For funzies. */
-    sock2 = socket(PF_INET, SOCK_STREAM, 0);
+    sock2 = priv_invoke_cap_fn(my_socket_handle, args);
     if (sock2 < 0) {
         perror("myecho(socket 2)");
         exit(EXIT_FAILURE);
